
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PG_CLT_XACTION_H
#define PG_CLT_XACTION_H

#include "base/ObjTimes.h"
#include "runtime/IOBuf.h"
#include "runtime/httpHdrs.h"
#include "runtime/Xaction.h"
#include "client/UserCred.h"

class Client;
class ServerRep;
class ContentCfg;
class DistrPoint;
class ObjWorld;
class HostCfg;
class MembershipMap;
class BodyParser;
class ParseBuffer;
class RegExGroup;
class RegExMatchee;

class CltXact: public Xaction {
	public:
		CltXact();

		virtual void reset();

		void enqueue();
		void exec(Client *anOwner, Connection *aConn);

		virtual void noteReadReady(int fd);
		virtual void noteWriteReady(int fd);

		virtual void cacheDistrPoint(DistrPoint *) {}
		virtual bool needRetry() const { return doRetry; }
		virtual void noteAbort();

		// called from BodyParsers
		void noteContent(const ParseBuffer &content);
		Error noteEmbedded(ReqHdr &hdr);
		void noteTrailerHeader(const ParseBuffer &hdr);
		void noteEndOfTrailer();

		bool askedPeer() const { return thePeerState != peerUnknown; }
		bool usePeer() const { return thePeerState == peerSome; }
		void usePeer(bool doUse) { thePeerState = doUse ? peerSome : peerNone; }

		const NetAddr &nextHop() const { return theNextHop; }
		NetAddr &nextHopVar() { return theNextHop; }

		const UserCred &credentials() const { return theCred; }
		void noteCredentialsGone(); // called by filters that stripped creds

		void saveRepHeader();
		const IOBuf &savedRepHeader() const { return theSavedRepHeader; }
		CltXact *cause() { return theCause; }
		const CltXact *cause() const { return theCause; }
		void cause(CltXact *aCause) { theCause = aCause; }
		int childCount() const { return theChildCount; }
		void noteChildNew(CltXact *) { theChildCount++; }
		void noteChildGone(CltXact *) { --theChildCount; }

	protected:
		virtual Agent *owner();
		virtual const UniqId &reqId() const;

		void makeConnectReq(WrBuf &buf);
		void makeExplicitReq(WrBuf &buf);
		void finishReqHdrs(WrBuf &buf, Size pfxSize, bool forceDump);

		Error getHeader();
		Error interpretHeader();
		void getBody();
		void parse();
		void checkOverflow();
		void noteError(const Error &err);
		void authWithProxy(bool needed);
		void redirect();

		virtual void finish(Error err);
		virtual void logStats(OLog &ol) const;

		void makeReqVersion(ostream &os);
		const HostCfg *makeReqHost(ostream &os);
		void makeReqMethod(ostream &os);
		virtual void makeEndToEndHdrs(ostream &os);
		void makeHopByHopHdrs(ostream &os);

		virtual void consume(Size size);
		virtual void firstHandSync();

		Error handleEmbedOid(ReqHdr &hdr);
		Error handleForeignEmbedOid(ReqHdr &hdr);

		bool expectMore() const;
		Size unconsumed() const;
		const ObjTimes &olcTimes() const;
		bool validRelOid(const ObjId &oid) const;
		bool cfgAbortedReply() const;

		void checkAcl();
		void checkAclMatch(const RegExGroup *matchGrp, const char *action);
		void buildAclMatchee(RegExMatchee &m) const;
		void explainAclMatch(const RegExMatchee &m, const char *action, const Array<RegExGroup*> &ourMatches) const;
		void dumpMatchingGroupNames(ostream &os, const MembershipMap *map) const;

		void checkFreshness();
		void checkDateSync();
		void updatePubWorld(const ObjWorld &slice);
		Error setStatusCode(int aStatus);
		Error setViserv(const NetAddr &name, ObjId &oid) const;
		BodyParser *selectBodyParser();
		BodyParser *selectContentParser();
		BodyParser *selectMarkupBodyParser();

	protected:
		Client *theOwner;
		ServerRep *theSrvRep;
		ContentCfg *theContentCfg;
		BodyParser *theBodyParser;
		NetAddr theNextHop;
		RepHdr theRepHdr;
		mutable ObjTimes theOlcTimes; // cached value
		Size theConsumedSize;
		Size theReqBodySize;
		UserCred theCred;
		IOBuf theSavedRepHeader; // to preserve raw response headers if needed
		CltXact *theCause;    // transaction that caused this transaction
		int theChildCount;    // transaction using us as a cause

		// this is used to check whether the xaction wants to use peers
		enum { peerUnknown = -1, peerNone, peerSome } thePeerState;

		bool doRetry;		// try the same xaction again
};

#endif
