
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */


#include "base/polygraph.h"

#include "xstd/h/iostream.h"
#include "xstd/h/strstream.h"

#include "xstd/gadgets.h"
#include "base/BStream.h"
#include "base/AggrStat.h"
#include "runtime/IOBuf.h"
#include "csm/ContentDbase.h"
#include "csm/cdbEntries.h"


/* CdbEntryPrnOpt */

CdbEntryPrnOpt::CdbEntryPrnOpt(): buf(0), injector(0), sizeMax(-1), entryOff(-1) {
}


/* CdbEntry */

void CdbEntry::pour(CdbEntryPrnOpt &opt) const {
	ostrstream os(opt.buf->space(), Min(opt.sizeMax, opt.buf->spaceSize()));
	print(os);
	opt.buf->appended(Size(os.tellp()));
}


/* ContentDbase */

ContentDbase::ContentDbase() {
}

ContentDbase::~ContentDbase() {
	while (theEntries.count()) delete theEntries.pop();
}

int ContentDbase::count() const {
	return theEntries.count();
}

double ContentDbase::entrySizeMean() const {
	const int cnt = count();
	if (!cnt)
		return -1;

	AggrStat stat;
	for (int i = 0; i < cnt; ++i)
		stat.record(theEntries[i]->size());
	return stat.mean();
}

static
IBStream &operator >>(IBStream &is, CdbEntry *&e) {
	const int type = is.geti();
	if (is.good()) {
		e = ContentDbase::CreateEntry(type);
		e->load(is);
	} else {
		e = 0;
	}
	return is;
}

static
OBStream &operator <<(OBStream &os, const CdbEntry *e) {
	Assert(e);
	os << e->type();
	e->store(os);
	return os;
}

void ContentDbase::load(IBStream &is) {
	theName = is.name();
	is >> theEntries;
}

void ContentDbase::store(OBStream &os) const {
	theName = os.name();
	os << theEntries;
}

void ContentDbase::add(CdbEntry *e) {
	theEntries.append(e);
}

ostream &ContentDbase::print(ostream &os) const {
	for (int i = 0; i < count(); ++i)
		theEntries[i]->print(os);
	return os;
}

CdbEntry *ContentDbase::CreateEntry(int type) {
	switch (type) {
		case cdbeBlob:
			return new CdbeBlob;
		case cdbeText:
			return new CdbeText;
		case cdbeTag:
			return new CdbeTag;
		case cdbeComment:
			return new CdbeComment;
		default:
			Assert(false); // unknown entry type
	}
	return 0;
}
