
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PG_INFORMATION_SCOPE_H
#define PG_INFORMATION_SCOPE_H

#include "xstd/String.h"
#include "xstd/Array.h"

class InfoScopeDim;


// manages scope rules/info
// scope is defined in terms of phase and side names
class InfoScope {
	public:
		InfoScope();
		InfoScope(const InfoScope &s);
		~InfoScope();

		void name(const String &aName);

		operator void*() const;

		String name() const;  // imprecise, human-readable
		String image() const; // precise, for machine use
		const Array<String*> &sides() const;
		const Array<String*> &phases() const;

		// narrow or set
		InfoScope oneSide(const String &name) const;
		InfoScope onePhase(const String &name) const;

		// expand
		void addSide(const String &name);
		void addPhase(const String &name);
		void add(const InfoScope &scope);

		// test
		bool hasSide(const String &name) const;
		bool hasPhase(const String &name) const;

		bool operator ==(const InfoScope &s) const;
		InfoScope &operator =(const InfoScope &s);

	protected:
		void reset();
		void copy(const InfoScope &s);

	protected:
		String theName;
		InfoScopeDim *theSides;
		InfoScopeDim *thePhases;
};

#endif
