
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include "xml/XmlTag.h"
#include "xml/XmlText.h"
#include "xml/XmlAttr.h"
#include "loganalyzers/ReportBlob.h"
#include "loganalyzers/BlobDb.h"
#include "loganalyzers/SomeInfo.h"


class MeasBlob: public ReportBlob {
	public:
		MeasBlob(const String &key);

		void value(const XmlAttr &val);
		void unit(const String &unit);

	public:
		XmlNode *measurement;
};

MeasBlob::MeasBlob(const String &key): ReportBlob(key), measurement(0) {
	static String tagName = "measurement";
	measurement = append(XmlTag(tagName));
}

void MeasBlob::value(const XmlAttr &val) {
	*measurement->attrs() << val;
}

void MeasBlob::unit(const String &unit) {
	static String attrName = "unit";
	*measurement->attrs() << XmlAttr(attrName, unit);
}


/* SomeInfo */

const ReportBlob &SomeInfo::addLink(BlobDb &db, const String &newKey, const String &oldKey) {
	ReportBlob blob(newKey);
	blob << db.include(oldKey);
	return *db.add(blob);
}

const ReportBlob &SomeInfo::addMeasBlob(BlobDb &db, const String &name, double val, const String &unit) {
	MeasBlob blob(name);
	blob.value(XmlAttr::Double("value", val));
	blob.unit(unit);
	return *db.add(blob);
}

const ReportBlob &SomeInfo::addMeasBlob(BlobDb &db, const String &name, Time val) {
	MeasBlob blob(name);
	blob.value(XmlAttr::Double("value", val.secd()));
	blob.unit("sec");

	XmlTag image("image");
	XmlText text;
	text.buf() << val;
	image << text;
	blob.measurement->addChild(image.clone());
	return *db.add(blob);
}

const ReportBlob &SomeInfo::addMeasBlob(BlobDb &db, const String &name, const String &val, const String &unit) {
	MeasBlob blob(name);
	blob.value(XmlAttr("value", val));
	blob.unit(unit);
	return *db.add(blob);
}

const ReportBlob &SomeInfo::addNaMeasBlob(BlobDb &db, const String &name) {
	static String key("na.measurement.reason=not_collected");
	static XmlText textNa("n/a");

	const ReportBlob *explBlob = db.has(key);
	if (!explBlob) {
		ReportBlob blob(key);
		blob << XmlAttr::Int("report_note", true);
		blob << XmlText("no measurement was collected or stored at run-time");
		explBlob = db.add(blob);
	}

	ReportBlob measBlob(name);
	measBlob << db.ptr(key, textNa);
	return *db.add(measBlob);
}
