
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#include "base/polygraph.h"

#include <iostream.h>

#include "xstd/gadgets.h"
#include "base/Histogram.h"
#include "base/TmSzHistStat.h"
#include "loganalyzers/Stex.h"
#include "loganalyzers/PhaseInfo.h"
#include "loganalyzers/TmSzHistFig.h"


TmSzHistFig::TmSzHistFig(): theStex(0), thePhase(0) {
}

void TmSzHistFig::stats(const Stex *aStex, const PhaseInfo *aPhase) {
	theStex = aStex;
	thePhase = aPhase;
}

void TmSzHistFig::setCtrlOptions() {
	ReportFigure::setCtrlOptions();
	*theCtrlFile << "set size 0.5, 0.5" << endl;
	*theCtrlFile << "set ylabel 'cumulative %'" << endl;
	*theCtrlFile << "set key left" << endl;
}

int TmSzHistFig::createCtrlFile() {
	if (ReportFigure::createCtrlFile() < 0)
		return -1;

	// collect all stexes to plot
	Array<const Stex*> stexes;
	for (const Stex *stex = theStex; stex; stex = stex->parent()) {
		if (stex->hist(*thePhase))
			stexes.append(stex);
	}

	// reverse stexes order so that the most inner one is on top
	for (int l = 0, r = stexes.count()-1; l < r; ++l, --r)
		stexes.swap(l,r);

	// create plot command
	{for (int s = 0; s < stexes.count(); ++s) {
		if (s == 0)
			*theCtrlFile << "plot '-' ";
		else
			*theCtrlFile << ",\\" << endl << "\t '-' ";
		*theCtrlFile << "title '" << stexes[s]->name() 
			<< "' with lines lt " << (stexes.count() - s);
	}}
	*theCtrlFile << endl;

	// dump data to plot
	int pointCount = 0;
	for (int s = 0; s < stexes.count(); ++s) {
		if (s)
			*theCtrlFile << 'e' << endl; // note: two empty lines do not work

		const TmSzHistStat *tmsz = stexes[s]->hist(*thePhase);
		Assert(tmsz);
		const Histogram &hist = extractHist(*tmsz);
		const int count = hist.stats().count();
		for (HistogramConstIter i(hist); count && i; ++i)
			pointCount += dumpDataLine(*i, count);

		if (theStex != stexes[s])
			pointCount = 0;
	}

	return pointCount;
}

int TmSzHistFig::dumpDataLine(const HistogramBin &bin, int totCount) {
	if (bin.count) {
		const double p = Percent(bin.accCount, totCount);

		if (p < 5)
			return 0; // ignore very small values

		if (Percent(bin.accCount-bin.count, totCount) > 95)
			return 0; // ingore very large values unless they contribute a lot

		*theCtrlFile << bin.sup << ' ' << p << endl;
		return 1;
	}
	return 0;
}

