
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef RUNTIME_PERSIST_WORK_SET_MGR_H
#define RUNTIME_PERSIST_WORK_SET_MGR_H

#include "base/UniqId.h"

class String;
class IBStream;
class OBStream;

// persistent working set
class PersistWorkSetMgr {
	public:
		PersistWorkSetMgr();
		~PersistWorkSetMgr();

		const UniqId &id() const;
		int version() const;

		void openInput(const String &anInFname);
		void openOutput(const String &anOutFname);

		void loadSeeds();
		void storeSeeds();

		void loadPubWorlds();
		void storePubWorlds();

		IBStream *loadSideState();  // may be null
		OBStream *storeSideState(); // may be null

		void checkInput();
		void checkOutput();

		void closeInput();
		void closeOutput();
		void close();

	protected:
		void loadHeader();
		void storeHeader();

		void loadMagic();
		void storeMagic();

		void loadTag(int expectedTag);
		void storeTag(int expectedTag);

	private:
		IBStream *theInStream;
		OBStream *theOutStream;
		UniqId theId;
		int theVersion;
};

extern PersistWorkSetMgr ThePersistWorkSetMgr;

#endif
