
/*
 * Web Polygraph                        http://www.web-polygraph.org/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. See the COPYRIGHT file for details.
 */

#ifndef PG_SERVER_CONFIGURATION_H
#define PG_SERVER_CONFIGURATION_H

#include "xstd/Array.h"
#include "csm/ContentTypeIdx.h"
#include "runtime/AgentCfg.h"

class XactAbortCoord;
class ServerSym;
class PopModel;
class RndDistr;
class Area;

// Server configuration items that can be shared among multiple servers
class SrvCfg: public AgentCfg {
	public:
		SrvCfg();
		~SrvCfg();

		void configure(const ServerSym *aServer);

		bool hasContType(int id) const { return theTypes.hasContType(id); }

		void selectAbortCoord(XactAbortCoord &coord);
		int pickEmbedContType(const ObjId &oid, const Area &category) const;

	protected:
		void configureRepTypes();

	public:
		const ServerSym *theServer;  // used to identify/share configs
		ContentTypeIdx theTypes;  // IDs of all known content types
		PopModel *thePopModel;    // popularity model to repeat oids
		RndDistr *theRepTypeSel;  // selects reply types	
		double theAbortProb;
};


class SrvSharedCfgs: protected Array<SrvCfg*> {
	public:
		SrvCfg *getConfig(const ServerSym *cfg);

	protected:
		SrvCfg *addConfig(const ServerSym *cfg);
};

#endif
